﻿/*
 * Основной файл ScoringSDK.
 *
 * API ядра:
 * ScoringSDK.setID - установит ID для работы SDK.
 * 
 * ID - строка с хэшом, выдается на каждую сборку игры. ID нужен чтобы получать обновления от сервера для конкретной игры.
 * ID является хэшом, чтобы исключить возможность перебора.
 * SDK проверяет данные в памяти и на сервере. На сервер также отправляется текущий URL для возможности фильтрации параметров для одной и тойже игры на разных доменах.
 * В случае получения новых настроек для оплаты - они сохраняются в памяти с ключом "config".
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    global.ScoringSDK = global.ScoringSDK || {};
    var config = global.ScoringSDK._config,
        utils = global.ScoringSDK._utils;

    var module = {
        _prefix: "config",
        _getDataFromMemory: function (key) {
            var data = global.ScoringSDK.load(this._prefix, key);
            if (!data) return false;
            try {
                utils.inherit(config, data);
            } catch (e) {
            }
        },
        _getDataFromServer: function (key) {
            var self = this,
                url = "http://prefixs.rankingames.com/scoring/game/info?key=" + key + "&url=" + encodeURIComponent(location.href);
            utils.ajax.get(url, function (data) {
                if (!data || !data.items) return false;
                utils.inherit(config, data.items);
                config.innap = config.innap || {};
                try {
                    global.ScoringSDK.save(self._prefix, config, key);
                } catch (e) {
                }
            });
        },
        setId: function (key) {
            if (typeof key != "string") return false;
            var self = this;
            self._getDataFromMemory(key);
            self._getDataFromServer(key);
        }
    };

    global.ScoringSDK.setID = function (key) {
        module.setId(key);
    };
})(this);