/*
 * Механизм иннапов для ScoringSDK.
 *
 * API модуля:
 * ScoringSDK.getInnap()  - получить объект иннапа. Ожидает один аргумент: callback-функцию.
 * ScoringSDK.checkInnap() - проверить объект иннапа. Ожидает два аргумента: объект иннапа и callback-функцию.
 * ScoringSDK.getUnlockedTime() - получить время демо-игры.
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    global.ScoringSDK = global.ScoringSDK || {};
    var config = global.ScoringSDK._config,
        utils = global.ScoringSDK._utils;

    var module = {
        _get: function (methodName, propertyName, defaultValue) {
            var value = (config.innap || {})[propertyName] || defaultValue || "";
            if (!global.API || !global.API[methodName]) return value;
            try {
                return global.API[methodName]() || value;
            } catch (e) {
                return value;
            }
        },
        getMcc: function () {
            return this._get("getMcc", "mcc", "250") || "";
        },
        getMnc: function () {
            return this._get("getMnc", "mnc", "01") || "";
        },
        getInnap: function (callback) {
            var url = config.innap.server + "get?priceGroup=" + (config.innap.priceGroup || "") + "&mcc=" +
                (config.innap.mcc || "") + "&mnc=" + (config.innap.mnc || "") + "&gameId=" + (config.innap.gameId || "");
            utils.ajax.get(url, function (data) {
                if (data && data.items && data.items.xml && data.items.xml) {
                    data = data.items.xml;
                }
                if (data.error) data = null;
                if (typeof callback == "function") callback(data);
            });
        },
        checkInnap: function (innap, callback) {
            innap = innap || {};
            var url = config.innap.server + "check?uid=" + (innap.id || "") + "&gameId=" + (config.innap.gameId || "");
            utils.ajax.get(url, function (data) {
                var status = false;
                if (data && data.status == "ok") status = true;
                if (typeof callback == "function") callback(status);
            });
        },
        getUnlockedTime: function () {
            return config.innap.unlockedTime;
        }
    };

    global.ScoringSDK.getInnap = function (callback) {
        return module.getInnap(callback);
    };

    global.ScoringSDK.checkInnap = function (innap, callback) {
        return module.checkInnap(innap, callback);
    };

    global.ScoringSDK.getUnlockedTime = function () {
        return module.getUnlockedTime();
    };
})(this);