/*
 * Набор инструментов для ScoringSDK.
 *
 * API модуля:
 * ScoringSDK._utils.inherit  - миксин двух JSON объектов. Ожидает два аргумента типа JSON.
 * ScoringSDK._utils.ajax.get - выполняет GET запрос на сервер. Ожидает URL и callback-функцию.
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    global.ScoringSDK = global.ScoringSDK || {};

    var utils = {
        inherit: function (object, property) {
            if (!object || !property) return false;
            for (var id in property) {
                if (typeof property[id] == "object") {
                    if (!object[id] || typeof object[id] != "object") object[id] = (property[id] instanceof Array) ? [] : {};
                    utils.inherit(object[id], property[id]);
                } else object[id] = property[id];
            }
        },
        addEvent: function (element, eventName, callback, context) {
            if (!element) return false;
            var self = this,
                realCallback = (context) ? function () {
                    callback.call(context, arguments);
                } : callback;
            if (element.addEventListener) {
                element.addEventListener(eventName, realCallback, false);
            } else if (element.attachEvent) {
                element.attachEvent('on' + eventName, realCallback);
            }
        },
        stopEvent: function (event) {
            if (!event) return false;
            if (event.stopPropagation) {
                event.stopPropagation();
            } else {
                event.cancelBubble = true;
            }
            if (event.preventDefault) {
                event.preventDefault();
            }
        }
    };

    utils.ajax = {
        _index: 0,
        _prefix: "__ScoringSDK__",
        _removeNode: function (parent, node, callbackName) {
            parent.removeChild(node);
            global[callbackName] = null;
        },
        _getParent: function () {
            var parent = document.head,
                getTag = document.getElementsByTagName,
                head = getTag.apply(document, [ "head" ]);
            if (!parent && head && head.length) return head[0];
            return parent;
        },
        _addEvent: function (parent, node, callbackName) {
            var self = this;
            utils.addEvent(node, "load", function () {
                self._removeNode(parent, node, callbackName);
            });
            utils.addEvent(node, "error", function () {
                if (typeof global[callbackName] == "function") {
                    global[callbackName]();
                }
                self._removeNode(parent, node, callbackName);
            });
        },
        get: function (url, callback) {
            if (typeof url != "string") return;
            var self = this,
                callbackName = self._prefix + self._index++,
                node = document.createElement("SCRIPT"),
                parent = self._getParent();
            global[callbackName] = callback;
            url += "&jsoncallback=" + callbackName;
            node.setAttribute("src", url);
            self._addEvent(parent, node, callbackName);
            parent.appendChild(node);
        }
    };

    global.ScoringSDK._utils = utils;
})(this);