﻿/*
 * Ядро фреймворка.
 * Содержит базовые функции для решения типовых задач (сменить класс у DOM элемента и т.д.)
 * Может выступать прослойкой между игровым фреймворком и любым другим, который обеспечить базовые функции взаимодействия с DOM.
 * 
 * Чтобы уменьшить вес фреймворка - можно заменить код базовых функций, на API другого фреймворка, который уже используется в проекте.
 * 
 * API ядра:
 * removeClass() - удалит класс. При вызове ожидает два аргумента: DOM элемент и имя класса
 * addClass() - добавит класс. При вызове ожидает два аргумента: DOM элемент и имя класса
 * toggleClass() - добавит / удалит класс. При вызове ожидает два аргумента: DOM элемент и имя класса
 * hasClass() - проверит наличие класса. При вызове ожидает два аргумента: DOM элемент и имя класса
 * addEvent() - повесит событие на элемент. При вызове ожидает три аргумента: DOM элемент, имя события, функция обратного вызова
 * stopEvent() - остановит всплытие события. При вызове ожидает один аргумент: событие
 * display.width() - вернет ширину окна браузера
 * display.height() - вернет высоту окна браузера
 * hasInArrayObjects() - проверит наличие объекта в массиве. При вызове ожидает три аргумента: массив, свойство по которому искать и значение, которое ищем. 
 * random() - вернет случайное число. При вызове ожидает два аргумента: минимум и максимум
 * shuffle() - перемешает массив
 * randomKey() - сгенерирует случайны хэш из латинских букв
 * inherit() - совершит наследование всех свойств родительского объекта - дочернему. При вызове ожидает два аргумента: объект наследник и объект родитель
 * isValidParams() - проверит валидность параметра
 * getValid() - проверит валидность параметра, если параметр не валиден - поставит ему значение по умолчанию
 * console() - выведит сообщение в консоль. При вызове ожидает один аргумент: сообщение. Нужно для того, чтобы иметь возможность контролировать все сообщения фреймворка.
 * 
 */
(function (global) {
    global = global || {};
    global.StalinGrad = global.StalinGrad || {};
    global.StalinGrad.module = global.StalinGrad.module || {};

    var DOMContentLoaded = [];
    var utils = {
        is: {
            android: (navigator.userAgent.toLowerCase().indexOf("android") > -1),
            operamini: (navigator.userAgent.toLowerCase().indexOf("opera mini") > -1),
            ie: (navigator.userAgent.toLowerCase().indexOf("msie") > -1),
            bada: (navigator.userAgent.search(/Bada/) > 0)
        },
        create: function (tagName, attrs, parentNode) {
            var node = document.createElement(tagName);
            attrs = attrs || {};
            for (var name in attrs) {
                var value = attrs[name];
                if (name == "innerHTML") {
                    node.innerHTML = value;
                    continue;
                }
                if (name == "className") {
                    utils.addClass(node, value);
                    continue;
                }
                node.setAttribute(name, value);
            }
            if (parentNode) {
                parentNode.appendChild(node);
            }
            return (node);
        },
        removeClass: function (node, className) {
            if (!node) return false;
            var removedClass = node.className;
            var pattern = new RegExp("(^| )" + className + "( |$)");
            removedClass = removedClass.replace(pattern, "$1");
            removedClass = removedClass.replace(/ $/, "");
            node.className = removedClass;
            return (true);
        },
        addClass: function (node, className) {
            if (!node) return false;
            if (!utils.hasClass(node, className)) {
                if (node.className == "") {
                    node.className = className;
                } else {
                    node.className += " " + className;
                }
            }
            return (true);
        },
        toggleClass: function (node, className) {
            if (!node) return false;
            return (utils[utils.hasClass(node, className) ? "removeClass" : "addClass"](node, className));
        },
        hasClass: function (node, className) {
            if (!node) return false;
            var pattern = new RegExp("(^| )" + className + "( |$)");
            return (!!node.className.match(pattern));
        },
        addEvent: function (element, eventName, callback, context) {
            if (!element) return false;
            var self = this,
                realCallback = (context) ? function () {
                    callback.call(context, arguments);
                } : callback;

            if (element.addEventListener) {
                if (self.is.android && eventName == "click") {
                    self.addEvent(element, "touchstart", function (event) {
                        utils.stopEvent(event);
                        realCallback(event);
                    });
                    self.addEvent(element, "touchmove", self.stopEvent);
                    self.addEvent(element, "touchend", self.stopEvent);
                } else {
                    element.addEventListener(eventName, realCallback, false);
                }
            } else if (element.attachEvent) {
                element.attachEvent('on' + eventName, realCallback);
            }
            if (global.test && global.test.addFunction) {
                global.test.addFunction(realCallback, eventName, element);
            }
        },
        ready: function (callback) {
            DOMContentLoaded.push(callback);
        },
        stopEvent: function (event) {
            if (!event) return false;
            if (event.stopPropagation) {
                event.stopPropagation();
            } else {
                event.cancelBubble = true;
            }
            if (event.preventDefault) event.preventDefault();
        },
        lock: {
            _timer: null,
            _locks: {},
            _isInit: false,
            _init: function () {
                var self = this;
                self._isInit = true;
                self._timer = setInterval(function () {
                    for (var index in self._locks) self._locks[index] = true;
                }, 300);
            },
            isFree: function (key) {
                var self = this;
                if (self._locks[key] === false) return false;
                if (self._locks[key] === true) {
                    self._locks[key] = false;
                    return true;
                }
                self._locks[key] = false;
                if (!self._isInit) self._init();
                return true;
            }
        },
        event: {
            events: [],
            getEvent: function (id) {
                for (var i = 0, l = this.events.length; i < l; i++) if (this.events[i].id == id) return i;
                return false;
            },
            addEvent: function (id) {
                this.events.push({ "id": id, "handlers": [] });
            },
            listen: function (eventName, callback) {
                if (!this.getEvent(eventName)) this.addEvent(eventName);
                var index = this.getEvent(eventName);
                if (index || index === 0) this.events[index].handlers.push(callback);
                if (global.test && global.test.addFunction) {
                    global.test.addFunction(callback, eventName);
                }
            },
            trigger: function (eventName, data) {
                var index = this.getEvent(eventName);
                if (index || index === 0) for (var i = 0, m = this.events[index], l = m.handlers.length; i < l; i++) {
                    var callback = (function (index, data) {
                        return function () {
                            try {
                                m.handlers[index](data);
                            } catch (e) {
                            }
                        }
                    })(i, data)
                    setTimeout(callback, 5);
                }
            },
            get: function () {
                function event() {
                    this._events = {};
                }

                event.prototype.listen = function (eventName, callback) {
                    if (!this._events[eventName]) this._events[eventName] = [];
                    this._events[eventName].push(callback);
                };
                event.prototype.remove = function (eventName) {
                    this._events[eventName] = [];
                };
                event.prototype.trigger = function (eventName, data) {
                    var arrayCallback = this._events[eventName] || [];
                    for (var i = 0, l = arrayCallback.length; i < l; i++) try {
                        arrayCallback[i] && arrayCallback[i](data);
                    } catch (e) {
                    }
                };
                return new event();
            }
        },
        display: {
            width: function () {
                if (window.innerWidth) {
                    windowWidth = window.innerWidth;
                } else if (document.documentElement && document.documentElement.clientWidth) {
                    windowWidth = document.documentElement.clientWidth;
                } else if (document.body.clientWidth) {
                    windowWidth = document.body.clientWidth;
                }
                return windowWidth;
            },
            height: function () {
                if (window.innerHeight) {
                    windowHeight = window.innerHeight;
                } else if (document.documentElement && document.documentElement.clientHeight) {
                    windowHeight = document.documentElement.clientHeight;
                } else if (document.body.clientHeight) {
                    windowHeight = document.body.clientHeight;
                }
                return windowHeight;
            }
        },
        hasInArrayObjects: function (arrayObjects, key, value) {
            if (!arrayObjects) return false;
            var numberInArray = false;
            for (var i = 0, l = arrayObjects.length; i < l; i++) {
                if (value == arrayObjects[i][key]) {
                    numberInArray = i;
                }
            }
            return numberInArray;
        },
        random: function random(min, max) {
            return Math.floor(Math.random() * (max - min + 1)) + min;
        },
        shuffle: function random(items) {
            for (var j, x, i = items.length; i; j = parseInt(Math.random() * i), x = items[--i], items[i] = items[j], items[j] = x);
            return items;
        },
        objectToArray: function (object) {
            var arrayElements = [];
            for (var index in object) arrayElements.push(object[index]);
            return arrayElements;
        },
        randomKey: function () {
            var keyLength = 5;
            var randomKey = "";
            var chars = "QWERTYUIOPLKJHGFDSAZXCVBNMqwertyuiopasdfghjklzxcvbnm".split("");
            for (var i = 0, l = chars.length; i < keyLength; i++) {
                var randomIndex = utils.random(0, (l - 1));
                randomKey += chars[randomIndex];
            }
            return randomKey;
        },
        inherit: function (children, parent) {
            var key = false;
            for (key in parent) {
                children[key] = parent[key];
                if (children[key] && children[key]._parent) {
                    children[key]._parent = children;
                }
            }
        },
        isValidParams: function (value) {
            if ((value || value === 0) && typeof value == "number") return true;
            return false;
        },
        getValid: function (value, defaultValue) {
            return ((utils.isValidParams(value)) ? value : defaultValue);
        },
        console: function (message) {
            console.log(message);
        },
        JSONP: function (global) {
            function JSONP(uri, callback) {
                function JSONPResponse() {
                    try {
                        delete global[src]
                    } catch (e) {
                        global[src] = null
                    }
                    documentElement.removeChild(script);
                    callback.apply(this, arguments);
                }

                var src = prefix + id++, script = document.createElement("script");
                global[src] = JSONPResponse;
                script.addEventListener("error", function () {
                    try {
                        throw "error loaded jsonp script";
                    } catch (e) {
                        documentElement.removeChild(script);
                        try {
                            callback.apply(this);
                        } catch (e) {
                        }
                    }
                });
                try {
                    var isEmptyParams = (uri[(uri.length - 1)] == "?" ? "" : "&");
                } catch (e) {
                    var isEmptyParams = "&";
                }
                uri += isEmptyParams + "lang=ru&rand=" + Math.random() + "&jsoncallback";
                documentElement.insertBefore(script, documentElement.lastChild).src = uri + "=" + src;
            }

            var id = 0, prefix = "__JSONP__", document = global.document, documentElement = document.documentElement;
            return JSONP;
        }(this)
    };

    var isOnLoaded = false,
        callbackOnLoad = function () {
            if (isOnLoaded) return false;
            isOnLoaded = true;

            for (var i = 0, l = DOMContentLoaded.length; i < l; i++) try {
                DOMContentLoaded[i] && DOMContentLoaded[i]();
            } catch (e) {
            }
            DOMContentLoaded = null;
        };
    if (document.body) document.body.onload = callbackOnLoad;
    utils.addEvent(document, "DOMContentLoaded", callbackOnLoad, false);

    utils.name = {
        _names: [],
        _newName: [],
        _oldName: [],
        get: function () {
            var self = this;
            if (self._newName.length < 1) self._newName = self._names.slice();
            var index = utils.random(0, (self._newName.length - 1));
            return self._newName.splice(index, 1)[0];
        },
        set: function (data) {
            this._names = data || [];
            this._newName = [];
            this._oldName = [];
        }
    };
    utils.name.set.prototype.get = utils.name.get;

    global.utils = utils;
    global.StalinGrad.utils = utils;
})(this);