/*
 * Аудио-модуль.
 * Задача модуля проигрывать звуки и создавать аудио сопровождение.
 *
 * Слушает следующие события:
 * audio__play  - зацикленное проигрывание трека. Ожидает один аргумент, а именно URL MP3 трека.
 * audio__play_once - проиграть трек один раз. Ожидает один аргумент, а именно URL MP3 трека.
 * audio__pause - пауза
 * audio__stop  - стоп
 *
 * Структура модуля:
 *   _node  - ссылка на DOM объект плеера
 *   _play  - событие Играть
 *   _pause - событие Пауза
 *   _stop  - событие Стоп
 *   _setEvents - установка событий
 *   init   - инициализация модуля. Срабатывает при загрузке страницы.
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    var utils = global.StalinGrad.utils;

    var module = {
        _node: null,
        _value: 100,
        _play: function (url, loop) {
            var self = this,
                node = self._node;
            if (!self._value) return false;
            if (global.API && global.API.audio) {
                global.API.audio(url);
            } else {
                if (!node) return false;
                if (loop) {
                    node.setAttribute("loop", "loop");
                } else {
                    node.removeAttribute("loop");
                }
                if (url && typeof url == "string") {
                    node.setAttribute("src", url);
                }
                if(node.play) node.play();
            }
        },
        _pause: function () {
            var node = this._node;
            if (node && node.pause) node.pause();
        },
        _stop: function () {
            var node = this._node;
            if (node && node.pause) node.pause();
        },
        _setValue: function (value) {
            this._value = value || 0;
        },
        _setEvents: function () {
            var self = this;
            utils.event.listen("audio__play", function (url) {
                self._play(url, true);
            });
            utils.event.listen("audio__play_once", function (url) {
                self._play(url, false);
            });
            utils.event.listen("audio__pause", function () {
                self._pause();
            });
            utils.event.listen("audio__stop", function () {
                self._stop();
            });
            utils.event.listen("audio__value_off", function () {
                self._setValue();
            });
            utils.event.listen("audio__value_on", function () {
                self._setValue(100);
            });
        },
        init: function () {
            var self = this;
            self._node = document.getElementById("audio");
            self._setEvents();
        }
    };

    utils.ready(function () {
        module.init();
    });
})(this);