﻿/*
 * Карусель кнопок
 * Карусель кнопок помогает создавать анимацию при переключение кнопок. Пример наиболее распростанненой проблемы, которую помогает решить этот модуль - кнопка играть/пауза.
 * В первом состоянии кнопка имеет значок треугольника и запускает музыку, во втором состоянии кнопка имеет вид двух полос и останавливает музыку.
 * Кроме смена атрибута src у картинки, этот модуль может делать замену текста в элемента при клике на них (либо замену аттрибута value для полей input).
 *
 * Пример использования модуля:
 * StalinGrad.module.carousel_buttons({
 *     id: "camera_information_button",                 // ID картинки, которая выступает в роли кнопки
 *     status: [                                        // Массив состояний
 *         {
 *             src: "images/module/pause.png",          // Адрес картинки
 *             title: "Остановить камеру",              // Подпись, либо текст в элементе
 *             callback: function() {                   // Функция, которая отработает при нажатии на кнопку
 *                 camera.pause();
 *             }
 *         },
 *         {
 *             src: "images/module/play.png",
 *             title: "Запустить камеру",
 *             callback: function() {
 *                 camera.play();
 *             }
 *         }
 *     ]
 * });
 *
 */
(function(global) {
	global = global || window;
	global.StalinGrad = global.StalinGrad || {};
	global.StalinGrad.module = global.StalinGrad.module || {};
	var utils = global.StalinGrad.utils;
	global = global.StalinGrad.module;

	function carousel(data) {
		var self = this;
		self.data = data.status;
		self.node = document.getElementById(data.id);
		self.index = 0;
		self.count = data.status.length - 1;
		if(!self.node || !self.data) return false;
		self._set();
	}

	carousel.prototype._set = function() {
		var self = this,
			node = self.node,
			item = self.data[self.index],
			text = item.text || item.value || item.title || false;
		if(node.src && item.src) node.setAttribute("src", item.src);
		if(node.className && item.className) node.className = item.className;
		if(text) {
			if(node.value) node.value = text;
			if(node.innerHTML) node.innerHTML = text;
			if(node.nodeName == "IMG") {
				node.setAttribute("alt", text);
				node.setAttribute("title", text);
			}
		}
	};

	carousel.prototype.back = function() {
		var self = this;
		self.index--;
		if(self.index < 0) self.index = self.count;
		self._set();
	};

	carousel.prototype.next = function() {
		var self = this;
		self.index++;
		if(self.index > self.count) self.index = 0;
		self._set();
	};

	carousel.prototype.click = function() {
		var item = this.data[this.index];
		item.callback && item.callback();
	};

	var callback = function(data) {
		if(data) {
			var object = new carousel(data),
				callback = function(event) {
					utils.stopEvent(event);
					object.click();
					object.next();
				};
			utils.addEvent(object.node, "click", callback);
			return object;
		}
	};

	global.carousel_buttons = callback;
})(window);