/*
 * Модуль памяти.
 * Нужен для сохранения и загрузки информации.
 * Если при загрузке или сохранение не задан ID ячейки памяти - работает с ячейкой "memory__auto"
 *
 * Слушает события:
 * memory__save - сохранить информацию. Ожидает объект с информацией и уникальным ID, по которому сохранять в память.
 * memory__load - загрузить информацию. Ожидает ID по которому будет искать информацию в памяти.
 *
 * Публикует события:
 * memory__data_received - событие публикуется, когда загружаются новые данные из памяти.
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    var utils = global.StalinGrad.utils;

    var module = {
        _showLog: false,
        _canSave: false,
        _prefix: "memory__",
        _index: 0,
        _setEvents: function () {
            var self = this;
            utils.event.listen("memory__save", function (data) {
                self.save(data);
            });
            utils.event.listen("memory__load", function () {
                return self.load();
            });
        },
        load: function () {
            var value = null;
            if (!global.API || !global.API.loadSomeThing) return false;
            try {
                if (global.API && global.API.loadSomeThing) {
                    value = global.API.loadSomeThing() || null;
                    if (global.JSON && global.JSON.parse && value) {
                        value = JSON.parse(value);
                    }
                }
            } catch (e) {
                if (this._showLog) console.log(e);
            }
            utils.event.trigger("memory__data_received", value);
            this._canSave = true;
            return value;
        },
        save: function (data) {
            data = data || {};
            var self = this;
            if (!self._canSave) return false;
            self._index++;
            if (!global.JSON || !global.JSON.stringify) return false;
            try {
                data = JSON.stringify(data);
                if (global.API && global.API.saveSomeThing) global.API.saveSomeThing(data, self._index);
            } catch (e) {
                if (this._showLog) console.log(e);
            }
        }
    };

    module._setEvents();
})(this);