/*
 * Модуль всплывающих окошек. Его задача заменить стандартный alert и confirm на красивые окна.
 * Слушает события:
 * message_show    - показать сообщение. Также ждет получение строки сообщения, которую он должен показать.
 * message_hide    - спрятать сообщение.
 * message_confirm - показать confirm. Ждет объект:
 *
 * {
 *     message: "Are you adult?"        // сообщение, которое будет показано
 *     ok: "Yes"                        // текст на кнопке согласия, по умолчанию ok
 *     cancel: "No"                     // текст на кнопке отмены, по умолчанию cancel
 *     callback: function(status) {}    // функция, которая будет вызвана после ответа пользователя. Функции будет передан статус выбора. TRUE - пользователь подтвердил вопрос. FALSE - пользователь отказался.
 * }
 *
 * Инициализируется по событию DOMContentLoaded
 *
 */

(function (global) {
    "use strict";

    global = global || window;
    global.StalinGrad = global.StalinGrad || {};
    global.StalinGrad.module = global.StalinGrad.module || {};
    var utils = global.StalinGrad.utils;
    global = global.StalinGrad.module;

    var module = {
        _nodes: {},
        _prefix: "message__",
        _callback: null,
        _isConfirm: false,
        _setMessage: function (data) {
            var node = this._nodes.info,
                message = (typeof data == "object") ? data.message : data;
            if (node) {
                node.innerHTML = message || "";
                this.show();
            }
        },
        _setButton: function (data) {
            var self = this,
                nodes = self._nodes,
                type = typeof data;
            if (nodes.ok) {
                nodes.ok.className = "message__button";
                if (type == "object" && data.ok) nodes.ok.innerHTML = data.ok;
            }
            if (nodes.cancel) {
                nodes.cancel.className = (self._isConfirm) ? "message__button" : "message__hidden";
                if (type == "object" && data.cancel) nodes.cancel.innerHTML = data.cancel;
            }
            self._callback = (self._isConfirm) ? data.callback : null;
        },
        show: function () {
            var node = this._nodes.table;
            if (node) node.className = "message__table";
        },
        hide: function () {
            var arrayID = [
                "table",
                "ok",
                "cancel"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                var node = this._nodes[(arrayID[i])];
                if (node) node.className = "message__hidden";
            }
            utils.event.trigger("message_hidden");
        },
        _setNodes: function () {
            var arrayID = [
                "table",
                "container",
                "close",
                "info",
                "ok",
                "cancel"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                this._nodes[(arrayID[i])] = document.getElementById((this._prefix + arrayID[i]));
            }
        },
        _setEventsClick: function () {
            var self = this,
                nodes = self._nodes,
                setEvent = function (id, type) {
                    utils.addEvent(nodes[id], "click", function (event) {
                        utils.stopEvent(event);
                        if (self._isConfirm) {
                            self._callback && self._callback((id == "ok"));
                        }
                        self.hide();
                    });
                };
            setEvent("table");
            setEvent("close");
            setEvent("cancel");
            setEvent("ok");
            utils.addEvent(nodes.container, "click", function (event) {
                utils.stopEvent(event);
            });
        },
        _setEventsListen: function () {
            var self = this;
            utils.event.listen("message_show", function (message) {
                self._isConfirm = false;
                self._setButton(message);
                self._setMessage(message);
            });
            utils.event.listen("message_confirm", function (data) {
                self._isConfirm = true;
                self._setButton(data);
                self._setMessage(data.message);
            });
            utils.event.listen("message_hide", function () {
                self.hide();
            });
        },
        init: function () {
            var self = this;
            self._setNodes();
            self._setEventsClick();
            self._setEventsListen();
        }
    };

    utils.ready(function () {
        module.init();
    });
})(this);