/*
 * Заставка.
 * Задача модуля показать заставку при загрузке страницы.
 * Модуль использует два таймера для того, чтобы событие о скрытие заставки было вызвано раньше фактического скрытия.
 * Это нужно для того чтобы модули, которые слушают данное событие, успели отреагировать.
 *
 * Структура модуля:
 *   _timer  <- ссылка на экземпляр таймера
 *     event <- таймер публикации события
 *     hide  <- таймер скрытия заставки
 *   _node   <- ссылка на DOM элемент заставки
 *   _setTimer  <- функция, устанавливает таймеры
 *   hide    <- функция, скрывает DOM элемент заставки
 *   init    <- функция инициализации
 *
 * Слушает события:
 * splash_screen__hide     - по событию скрывает заставку
 * splash_screen__set_time - по событию запускает таймер заставки.
 *
 * Публикует события:
 * splash_screen__hidden   - информирует о том, что заставка собирается скрыться через 300 миллисекунд
 *
 */
(function (global) {
    "use strict";

    global = global || {};
    var utils = global.StalinGrad.utils;

    var module = {
        _timer: {
            event: null,
            hide: null
        },
        _prefix: "splash_screen__",
        _nodes: {},
        _setTimer: function (time) {
            var self = this,
                timer = self._timer;
            time = time || 3000;

            clearTimeout(timer.event);
            timer.event = setTimeout(function () {
                utils.event.trigger("splash_screen__hidden");
            }, (time - 300));

            clearTimeout(timer.hide);
            timer.hide = setTimeout(function () {
                self.hide();
            }, time);
        },
        hide: function () {
            var node = this._nodes.table;
            if (node) node.className = "splash_screen__hidden";
        },
        _setEvents: function () {
            var self = this;
            utils.event.listen("splash_screen__set_time", function (time) {
                self._setTimer(time);
            });
        },
        _setNodes: function () {
            var arrayID = [
                "table",
                "image"
            ];
            for (var i = 0, l = arrayID.length; i < l; i++) {
                this._nodes[(arrayID[i])] = document.getElementById((this._prefix + arrayID[i]));
            }
        },
        _resize: function () {
            var node = this._nodes.image,
                w = utils.display.width(),
                h = utils.display.height(),
                min = Math.ceil(Math.min(w, h) * 0.8);
            if (node) node.style.width = min + "px";
        },
        init: function () {
            var self = this;
            self._setNodes();
            self._setTimer();
            self._setEvents();
            self._resize();
        }
    };

    utils.ready(function () {
        module.init();
    });
})(this);