﻿/*
 * Тестирование и документирование кода.
 * Модуль для построения карты объекта. Ждет на входе объект или функцию и строит карту свойств и методов.
 * Модуль нужен для того, чтобы можно было наглядно посмотреть на архитектуру модуля, описать его работу, перепроверить API.
 *
 * Пример вызова:
 *   StalinGrad.utils.test.map(object);
 * Вернет строку (также выведет её в консоль):
 *   module <- object
 *       filters <- object
 *       node <- object
 *           group <- object
 *           elements <- object
 *       lastID <- object
 *
 * Пример вызова:
 *   StalinGrad.utils.test.map(function);
 * Вернет строку (также выведет её в консоль):
 *   function
 *     property: <- object
 *       id <- string
 *     prototypes: <- object
 *       m1 <- function
 *       m2 <- function
 */
(function (global) {
    "use strict";

    global = global || {};
    global.test = global.test || {};

    var module = {
        _addRecommendations: function () {
            var self = this,
                statistics = self._statistics,
                recommendations = [
                    "Max depth search: ", (statistics.maxDepthSearch - 1), "\r\n",
                    "Number methods: ", (statistics.numberMethods - 1), "\r\n",
                    "Number internal API: ", (statistics.internalAPI), "\r\n",
                    "Number external API: ", (statistics.externalAPI - 1), "\r\n",
                    "External API: ", ("\r\n    " + statistics.externalApiDoc.join("\r\n    "))

                ];
            self._fulltext += "\r\n" + recommendations.join("");
        },
        _clearStatistics: function () {
            this._fulltext = "";
            this._statistics = {
                maxDepthSearch: 0,
                numberMethods: 0,
                externalAPI: 0,
                internalAPI: 0,
                externalApiDoc: []
            }
        },
        _print: function (name, type, depthSearch) {
            var text = "";
            for (var i = 0; i < depthSearch; i++) text += "    ";
            this._fulltext += text + name + " <- " + type + "\r\n";
        },
        _checkObject: function (object, name, depthSearch) {
            var self = this;
            name = name || "module";
            depthSearch = depthSearch || 0;
            self._statistics.numberMethods++;
            if (name[0] == "_") {
                self._statistics.internalAPI++;
            } else {
                self._statistics.externalAPI++;
                if (name != "module") self._statistics.externalApiDoc.push(name);
            }
            if (typeof object == "object") {
                if (object instanceof Array) {
                    self._print(name, "array", depthSearch);
                } else {
                    self._print(name, "object", depthSearch);
                    depthSearch++;
                    if (depthSearch > self._statistics.maxDepthSearch) self._statistics.maxDepthSearch = depthSearch;
                    for (var method in object) {
                        self._checkObject(object[method], method, depthSearch);
                    }
                    depthSearch--;
                }
            } else {
                self._print(name, typeof object, depthSearch);
            }
        },
        _checkFunction: function (object) {
            var self = this,
                children = new object();
            children.__proto__ = {};
            self._checkObject(children, "  property:");
            self._checkObject(object.prototype, "  prototypes:");
        },
        get: function (data, name) {
            var self = this;
            self._clearStatistics();
            if (typeof data == "function") {
                self._fulltext = (name || "function") + "\r\n";
                self._checkFunction(data, name);
            } else {
                self._checkObject(data, name);
            }
            self._addRecommendations();
            console.log(self._fulltext);
            return (self._fulltext.replace(/(\r\n)$/gim, "") || "");
        }
    };

    global.test.doc = function (object, name) {
        try {
            return module.get(object, name);
        } catch (e) {
            return false;
        }
    };
})(this);